% Compute the Coriolis (traditionally known as gyroscopic) matrices of the 
% individual beam gcells. 
% Return an element set of them so they may be assembled.
%    
% function ems = coriolis (self, geom0, u1, Rfield1, v1, dchi)
% 
%     geom0=initial geometry field; locations of nodes
%     u1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     v1=current velocity field; 
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.   
%
% The  matrices are computed in the local coordinate system and
% then transformed into the global coordinate system. 
%
% These matrices are obtained by consistent linearization of the kinetic
% energy term with the the element mass matrix assumed constant in
% the element frame.
function ems = coriolis (self, geom0, u1, Rfield1, v1, dchi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom0,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Me = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(dchi,conns(i,:),'eqnums');
        Me{i} =zeros(6*nfens);
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    Rs = gather(Rfield1, (1:get(geom0,'nfens')), 'values', 'noreshape'); % displacement in step n
    V1s =gather(v1,(1:get(v1,'nfens')),'values','noreshape');
    X1s=Xs+U1s;
   % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );
    nu=get(prop , 'nu' );
    G = E/2/(1+nu);
    rho=get(prop , 'rho' );
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I1=get(gcells,'I1');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    J=get(gcells,'J');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);% current directors at node 1
        RJ =reshape(Rs(conn(2),:),3,3);% current directors at node 2
        v1 = V1s(conn,:); % current coordinates of nodes
        [L1,T,d_N]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        v1f= [v1(1,1:3)*T,v1(1,4:6)*T;v1(2,1:3)*T,v1(2,4:6)*T]; %(T'*v1(1,1:3)')'
        Omega=mean(v1f(:,4))*T(:,1)...
            +(v1f(1,3)-v1f(2,3))/L1*T(:,2)...
            +(v1f(2,2)-v1f(1,2))/L1*T(:,3);
        OS=skewmat(Omega);
        L0 =norm((x0(2,:)-x0(1,:)));
        Me{i}=local_mass(self, A(i), I1(i), I2(i), I3(i), rho, L0);
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        OmegaTilde=[OS Z Z Z; ...
                    Z OS Z Z;...
                    Z Z OS Z;...
                    Z Z Z OS];
        Me{i} = OmegaTilde*Te*Me{i}*Te'-Te*Me{i}*Te'*OmegaTilde;
    end
     ems  = elematset(struct('mat',{Me}, 'eqnums',{eqnums}));
end
