% Compute the element load vectors corresponding to prescribed heat flux. 
%
% function evs = flux_loads (self, geom, temp, fi)
%
% Return an array of the element loads so they may be assembled.
%    Call as
% evs = flux_loads(feb, geom, temp, fi)
%     geom=geometry field
%     temp=temperature field
%     fi=magnitude of the flux (force intensity)

% The heat flux is positive when the heat leaves the body, and negative when it enters.
%
function evs = flux_loads (self, geom, temp, fi)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Fe{i} =zeros(nfens,1);
    end
    xs =get(geom,'values');% Retrieve Geometry information
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            f=get_magn(fi, Ns{j}'*x); 
            Fe{i} = Fe{i}  - (Jac*w(j)*f)*Ns{j};
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
