% Set the EBC's (essential boundary conditions).
%
% function retobj = set_ebc (self, fenids, is_prescribed, comp, val)
%
% To actually apply the EBC's (that is to copy the prescribed values
% into the appropriate places in the values array), use apply_ebc().
%    Call as:
%       retobj = set_ebc (self, fenids, is_prescribed, comp, val)
%    where
%       self           - field
%       fenids         - array of N node identifiers
%       is_prescribed  - an array of N Boolean values which say whether the component
%                        is being prescribed (value ~=0), or whether
%                        it is to be free (value ==0).
%       comp           - an array of N components to which to apply the EBC's; comp
%                        may be supplied as '[]', in which case the
%                        prescribed values apply to all components.
%       val            - an array of N values of prescribed component magnitudes,
%                        length(val) == length(fenids)
%
% Note:  Any call to set_ebc() that would change the current assignment 
% which degrees of freedom are free and which are prescribed invalidates the
% current equation numbering. In such a case this method sets 
%    neqns = 0; and 
%    eqnums=eqnums*0+get(sysmat, 'invalid_eqnum'); 
%
function retobj = set_ebc (self, fenids, is_prescribed, comp, val)
    previous_is_prescribed=self.is_prescribed;
    Maxl =max([length(fenids), length(is_prescribed), length(comp), length(val)] );
    if (length(fenids)==1)&&(Maxl>1)
        fenids=repmat(fenids,Maxl,1);
    end
    if (length(is_prescribed)==1)&&(Maxl>1)
        is_prescribed=repmat(is_prescribed,Maxl,1);
    end
    if (length(comp)==1)&&(Maxl>1)
        comp=repmat(comp,Maxl,1);
    end
    if (length(val)==1)&&(Maxl>1)
        val=repmat(val,Maxl,1);
    end
    [nfens,dim] = size(self.values);
    n = length(fenids);
    for i=1:n
        k = fenids(i);
        if (isempty(comp))
            m = 1:dim;
        else
            m = comp(i);
            if m>dim
                error(['Component: ' num2str(m) ' versus dim=' num2str(dim)]);
            end
        end
        self.is_prescribed(k,m) = is_prescribed(i);
        if (self.is_prescribed(k,m))
            self.prescribed_values(k,m) = val(i);
        end
    end
    % Any call to set_ebc() which changes which degrees of 
    %     freedom are free and which are prescribed
    %     invalidates the current equation numbering.
    changed=find(self.is_prescribed~=previous_is_prescribed);
    if (~ isempty(changed))
        self.neqns = 0;
        self.eqnums=self.eqnums*0+get(sysmat, 'invalid_eqnum');
    end
    % Return the modified object
    retobj = self;
end
