% Assemble an array of element matrices.
%
% function retobj = assemble (self, ems)
%
% Assemble a cell array of element matrices, that is objects of type elematset,
% into a sparse matrix.
%
function self = assemble (self, ems)
    % First find out if all matrices are symmetric
    all_symmetric=true;
    ne =length(ems);
    for m = 1:ne
        if ~placesymm(ems(m))
            all_symmetric=~true;
            break;
        end
    end
    if (all_symmetric)
        self = assemble_symm(self, ems);% only for symmetric matrices
    else
        self = assemble_unsymm(self, ems);% general but slow
    end
end



function [sppatti, sppattj] = conn_to_sparse(conns,neqns)
    ne=size(conns,1);
    nn=size(conns,2);
    rowp = cell(neqns,1);
    for i=1:ne
        c=unique(conns(i,:));
        for j=1:length(c)
            ij=c(j);
            rowp{ij}= [rowp{ij} c];
        end
    end
    sl=0;
    for i=1:neqns
        rowp{i}= unique(rowp{i});
        sl=sl+length(rowp{i});
    end
    sppatti= zeros (1,sl);
    sppattj= zeros (1,sl);
    k=1;
    for i=1:neqns
        r= rowp{i};
        rn=length(r);
        if ~isempty (r)
            sppatti(k:k+rn-1) =i;
            sppattj(k:k+rn-1) =r;
            k = k+rn;
        end
    end
    sppatti=sppatti(1:k-1);
    sppattj=sppattj(1:k-1);
    return;
end

function self = assemble_symm (self, ems)
    invalid_eqnum = get(self.sysmat, 'invalid_eqnum');
    [neqns,neqns] = size(self.mat);
    ne = length(ems);
    sl = 0;
    for m = 1:ne
        eqnums = get(ems(m), 'eqnums');
        for j = 1:length(eqnums)
            sl = sl + length(eqnums{j})^2;
        end
    end

    % There are two possible methods of assembling the sparse matrix, a
    % fast version, which uses more memory, and a slower, but more memory
    % efficient version. 
    %
    % Currently an arbitrary memory limit is used to determine which method
    % is appropriate. We could perhaps base this limit on the output from
    % the 'memory' command in the future and some guess of the required
    % array size
    if neqns < 40000
        % build the matrix the fast way
        
        % preallocate the vectors to hold the indices of the sprase matrix
        % that will be created to hold the information
        sppatti = zeros (1,sl);
        sppattj = zeros (1,sl);

        % Preallocate a vector to hold the values for the sparse matrix
        Ainit = zeros (1,sl);

        startind = 1;
        for m = 1:ne
            eqnums = get(ems(m), 'eqnums');
            mat    = get(ems(m), 'mat');
            for j = 1:length(mat)
                emask = eqnums{j} > invalid_eqnum;
                conns = eqnums{j}(emask);
                rmat = mat{j}(emask,emask);
                connlen = length(conns);
                endind = (startind+(connlen*connlen)-1);
                % put the contents of rmat into the Ainit array
                Ainit(startind:endind) = rmat;
                tempinds = repmat(conns(1:connlen), 1, connlen);
                sppatti(startind:endind) = reshape(tempinds', [], 1);
                sppattj(startind:endind) = reshape(tempinds, [], 1);
                startind = startind + (connlen * connlen);
            end
        end
        self.mat = self.mat + sparse(sppatti(1:endind), sppattj(1:endind), Ainit(1:endind), neqns, neqns);
    else
        % build the matrix the slow but memory efficient way
        
        % preallocate the sparse matrix so we can build it efficiently and
        % avoid out-of memory errors
        sparsesymm = spalloc(neqns, neqns, sl);

        for m = 1:ne
            eqnums = get(ems(m), 'eqnums');
            mat    = get(ems(m), 'mat');
            for j = 1:length(mat)
                emask = eqnums{j} > invalid_eqnum;
                conns = eqnums{j}(emask);
                rmat = mat{j}(emask,emask);
                connlen = length(conns);
                tempinds = repmat(conns(1:connlen), 1, connlen);
                idx = [ reshape(tempinds', [], 1), reshape(tempinds, [], 1)];
                for i = 1:size(idx,1)
                    sparsesymm(idx(i,1), idx(i,2)) = sparsesymm(idx(i,1), idx(i,2)) + rmat(i);
                end
            end
        end
        self.mat = self.mat + sparsesymm;
    end
    
    self.initialized = true;
    
end


function self = assemble_unsymm (self, ems)
    invalid_eqnum = get(self.sysmat, 'invalid_eqnum');
    [neqns_row,neqns_col] = size(self.mat);

    ne =length(ems);
    sl=0;
    for m = 1:ne
        eqnums_row=get(ems(m), 'eqnums_row');
        eqnums_col=get(ems(m), 'eqnums_col');
        for j=1:length(eqnums_row)
            sl =sl+length(eqnums_row{j})*length(eqnums_col{j});
        end
    end
    sppatti= zeros (1,sl);
    sppattj= zeros (1,sl);
    Ainit= zeros (1,sl);
    triplet =1;
    for m = 1:ne
        eqnums_row=get(ems(m), 'eqnums_row');
        eqnums_col=get(ems(m), 'eqnums_col');
        mat    = get(ems(m), 'mat');
        for j=1:length(mat)
            emask_row=eqnums_row{j}>invalid_eqnum;
            emask_col=eqnums_col{j}>invalid_eqnum;
            conns_row=eqnums_row{j}(emask_row);
            conns_col=eqnums_col{j}(emask_col);
            rmat=mat{j}(emask_row,emask_col);
            for r= 1:length(conns_row)
                for c= 1:length(conns_col)
                    sppatti(triplet) =conns_row(r);
                    sppattj(triplet) =conns_col(c);
                    Ainit(triplet) =rmat(r,c);
                    triplet =triplet +1;
                end
            end
        end
    end
    self.mat=self.mat+sparse(sppatti(1:triplet-1), sppattj(1:triplet-1), Ainit(1:triplet-1), neqns_row,neqns_col);
    self.initialized = true;
end
