%% Buckling of shaft under torsion, clamped-simple support
% 

%% 
% Description:
% Shaft under torsion, circular cross-section
% Case 4 from Table 2.1 of J.H.  ARGYRIS,   P.C.  DUNNE  and  D.W. SCHARPF, ON LARGE  
% DISPLACEMENT-SMALL   STRAIN  ANALYSIS   OF  STRUCTURES 
% WITH  ROTATIONAL   DEGREES  OF  FREEDOM. COMPUTER  METHODS  
% IN APPLIED  MECHANICS  AND  ENGINEERING   14  (1978)  401-451 .
% Clamped at the bottom
% Simply-supported at the top  -- buckling factor for semi-tangential moment 2.168,
% buckling factor for quasi-tangential moment 1.576

% Parameters:
E=10000;
nu=0.3;
L= 42; % Length of the beam
R = 1/2; % radius
scale=100;
magn=pi*E*(pi*R^4/4)/L;

neigvs = 1;

%% 
% Mesh
n=6;
tolerance=L/n/100;
Parameters=struct ('A',pi*R^2,'I2',pi*R^4/4,'I3',pi*R^4/4,'J',pi*R^4/2,...
        'x1x2_vector',[0 1 0], 'label',1);
[fens,gcells] = Beam_member([0,0,0;0,0,L],n,@gcellset_beam3,Parameters);

% The bottom end 
bottomn=fenode_select(fens, struct('box', update_box([],[0,0,0]), 'inflate', tolerance));
% The top end
topn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));

prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_beam3(struct ('mater',mater, 'gcells',gcells));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

% Apply EBC's
% Bottom end 
ebc_fenids=[bottomn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
% Top end 
ebc_fenids=(topn)*[1,1];
ebc_prescribed=[1,1];
ebc_comp=[1,2];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

% Assemble the system matrix
K = start (dense_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur));

%% 
% The critical load depends on the application of the torque. Here we
% compute it for the quasi-tangential and semi-tangential moments:
for kind= { 'semi-tangential','quasi-tangential'}
    
    % Load
    nl=nodal_load(struct ('id',topn,'dir',[6],'magn',[magn],...
        'kind', kind{1},'data',struct('lever_direction', 1)));
    F = sysvec;
    F = start (F, neqns);
    evs = loads(nl, ur);
    F = assemble (F, evs);
    % Solve
    ur = scatter_sysvec(ur, K\F);
    % get(ur,'values')
    
    
    % Plot
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=20;
    u=slice(ur,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
    draw_axes(gv, struct([]));
    
    %%
    % Compute the geometric stiffness matrix
    Kg = start (sparse_sysmat, neqns);
    Kg = assemble (Kg, cat(2,stiffness_geo(feb, geom, ur),...
        stiffness_moment(feb, geom, ur, nl)));
    
    
    %%
    % Solve the buckling problem
    [W,Omega]=eigs(-(get(Kg,'mat')+get(Kg,'mat')')/2,(get(K,'mat')+get(K,'mat')')/2,...
        neigvs,'lm');
    Omegas=(1./(diag(Omega)));
    
    disp(['Torque ' kind{1} ' '])
    for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
    end
    disp(' ')
    
    
    %%
    % Present the buckling modes
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=5;
    w=clone(ur,'w'); % make a copy of u
    for i=1:neigvs
        ur = scatter_sysvec(w, W(:,i));
        u=slice(ur,(1:3),'u');
        rot=slice(ur,(4:6),'rot');
        wv = get(u,'values');
        wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        gv=reset (gv,[]);
        draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
        draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot,'colorfield',colorfield));
        draw_axes(gv, struct([]));
        title(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
        pause(2); % next eigenvector
    end
    
end
