%% Buckling of a stepped column.
% 
% A stepped beam is clamped at one and, axially loaded by two forces:
% tensile force at the free and and a compressive load on the flat step
% face of the junction of the two cross-sections. 
% Reference: Warren C. Young, Roark's formulas for stress and strain,
% McGraw-Hill, sixth edition, Table 34, case 2a page 672.
% Lowest buckling parameter 22.5 (for tensile force 1000 N, compressive force 2000 N)
% 
function CB_stepped_column
%%
% Material parameters (steel).
E=2e5;% MPa=2e11 Pa
nu=0.3;

%%
% Dimensions
L1=200;% millimeter
L2=100;% millimeter
%%
% Cross-sectional properties
Parameters1=Beam_cross_section(struct('type','circle','R',11.982/2));% millimeters
Parameters2=Beam_cross_section(struct('type','circle','R',10./2));% millimeters


%%
% Simulation control parameters.
graphics=~true;


%%
% The mesh.
n=64;
X= [0,0,0; 0,0,L1; 0,0,L1+L2];
tolerance=L2/1e4;
Parameters1.x1x2_vector=[1 0 0];
[fens,gcells] = Beam_member(X(1:2,:),n,@gcellset_beam3,Parameters1);
Parameters2.x1x2_vector=[1 0 0];
[fens1,gcells1] = Beam_member(X(2:3,:),n,@gcellset_beam3,Parameters2);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tolerance);
gcells= cat(gcells1,gcells2);
bottomn=fenode_select(fens, struct('box', update_box([],X(1,:))));
midn=fenode_select(fens, struct('box', update_box([],X(2,:))));
topn=fenode_select(fens, struct('box', update_box([],X(3,:))));

%%
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%%
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));


%%
% Construct the requisite fields, geometry and displacement
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield0 = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=bottomn;
ebc_prescribed=[1];
ebc_comp=1:6;% clamped
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

%%
% Solver

if (graphics )
    gv=graphic_viewer;
end


%% 
% Forces
topnl=nodal_load(struct ('id',topn,'dir',[3],'magn',[1000]));
midnl=nodal_load(struct ('id',midn,'dir',[3],'magn',[-2000]));
F = start (sysvec, neqns);
F = assemble (F, loads(topnl, dchi));
F = assemble (F, loads(midnl, dchi));


%% 
% Stiffness matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb,geom0,u0,Rfield0,dchi));


%% 
% Linear analysis to determine internal forces.
dchi = scatter_sysvec(dchi, K\F); % Displacement increment

%% 
% Update the geometry
u1=u0+slice(dchi,(1:3),'transl');
Rfield1=update_rotation_field(Rfield0,slice(dchi,(4:6),'rotat'));

%% 
% Compute geometric stiffness
Kg = start (sparse_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb,geom0,u1,Rfield1,dchi));


%%
% Buckling analysis
options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eig(-full((get(Kg,'mat')+get(Kg,'mat')')/2),full(get(K,'mat')+get(K,'mat')')/2);
Omegas=min(1./max(diag(Omega)));

disp([' Buckling parameter=' num2str(Omegas) ', compared with the analytic 22.5, ' num2str(Omegas/22.5*100) '%'])

if (graphics )
    %%
    % Graphic display of the mesh and the deformed beam
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom0, 'u', u0, 'R', Rfield0, 'facecolor','none'));
    dchi = scatter_sysvec(dchi, W(:,1));
    scale=L1/10/norm(slice(dchi,(1:3),'transl'));
    wu1 = u1 + scale*slice(dchi,(1:3),'transl');   % increment displacement
    wRfield1=update_rotation_field(Rfield1,scale*slice(dchi,(4:6),'rotat'));
    draw(feb, gv, struct ('x', geom0, 'u', wu1, 'R', wRfield1, 'facecolor','yellow'));
    
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw_axes(gv, struct([]));
    camset (gv, [-191.7314 -234.2888  331.4447   32.8952   58.4503  118.4085         0         0    1.0000   30.5575]);
    pause(1)
end

