%% Prestressed simply-supported beam modal analysis
% 
% Presents the influence of pre-stress on the natural
% frequency. This is the classical curve that shows how the frequency drops
% to zero as we approach the Euler buckling load, and all the other hand
% frequency increases as we apply tensile force.

%% 
% Parameters:
E=30002*1000;%30002ksi
nu=0.;
b=18; h=18; L=300;% in, cross-section and length
g=32.17*12; % in/second^2
rho=0.2/g;% Pound/g mass density per unit volume

%%
% Cross-sectional properties
Parameters=Beam_cross_section(struct('type','rectangle','d2',b,'d3',h));

%% 
% Choose the mass formulation:
consistent_no_rotation_inertia=0;
consistent_with_rotation_inertia=1;
lumped_diagonal_no_rotation_inertia=2;
lumped_diagonal_with_rotation_inertia=3;
mass_type=2;

%% 
% Analytical frequencies 
% 
% The analytical frequencies were taken from table 8-1 of Formulas for
% natural frequency and mode shape, Robert D. Blevins, Krieger publishing
% company, Malabar Florida, reprint edition 2001.
% 
% The beam has cylindrical supports at either end. Therefore in the
% vertical plane it behaves as a simply supported beam, while in the
% horizontal plane it behaves as a clamped beam.
% 
% Vertical plane, simply supported:
analyt_Frequencies(1)=(1*pi)^2/(2*pi*L^2)*sqrt(E*Parameters.I2/rho/Parameters.A);
neigvs=1;


%% 
% The critical Euler buckling load (simple-support): (pi^2*E*I2/L^2).
% Carry out the calculation of the modified frequency for
PEul=(pi^2*E*Parameters.I2/L^2);
Ps=linspace(-1, 1, 100);

%% 
% Select the number of elements per half the length.
n=4;
Parameters.x1x2_vector=[1 0 0];
[fens,gcells] = Beam_member([[0 -L/2 0];[0 L/2 0]],2*n,@gcellset_beam3,Parameters);
tolerance=L/n/100;

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells,'mass_type',mass_type));


%%
% Construct the requisite fields, geometry and displacement
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield0 = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 -L/2 0]),'inflate', tolerance));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5,6];% cylindrical
ebc_val=0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L/2 0]),'inflate', tolerance));
ebc_prescribed=[1];
ebc_comp=[1,3,5,6];% cylindrical
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

%% 
% Assemble the global discrete system

% Assemble the system stiffness matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb,geom0,u0,Rfield0,dchi));
% Load
F = sysvec;
F = start (F, neqns);
evs = loads(nodal_load(struct ('id',fenode_select(fens, struct('box', update_box([],[0 L/2 0]),'inflate', tolerance)),'dir',[2],'magn',[1])), dchi);
F = assemble (F, evs);
% Solve for the displacement under the static load
dchi = scatter_sysvec(dchi, K\F);


%% 
% Update deflections so that the initial stress can be computed.
u1 = u0 + slice(dchi,(1:3),'transl');   % increment displacement
Rfield1=update_rotation_field(Rfield0,slice(dchi,(4:6),'rotat'));
        
%% 
% The static deflection is now used to compute the internal forces
% which in turn lead to the geometric stiffness.
Kg = start (sparse_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb,geom0,u1,Rfield1,dchi));

%  The mass matrix is computed next.
M = start (sparse_sysmat, neqns);
M = assemble (M, mass(feb,geom0,u0,Rfield0,dchi));

%% 
% The prestress-modified natural frequencies are computed in the loop for 
% a number of prestress force values.
ModifiedFrequencies=[];
for P=Ps*PEul
    options.tol =1e-26;
    options.maxit = 5000;
    options.disp = 0;
    [W,Omega]=eigs(get(K,'mat')+P*get(Kg,'mat'),get(M,'mat'), neigvs,'sm', options);
    [Omegas2,ix]=sort((diag(Omega))) ;
    Omegas2=somel(diag(Omega),ix);
    Frequencies= sqrt(Omegas2)/2/pi;
    for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) ' (analyt=' num2str(analyt_Frequencies(i)) ') [Hz]']);
    end
    ModifiedFrequencies= [ModifiedFrequencies, Frequencies(1)];
end

%% 
% Graphical presentation of the influence of pre-stress on the natural
% frequency. This is the classical curve that shows how the frequency drops
% to zero as we approach the Euler buckling load, and all the other hand
% frequency increases as we apply tensile force.
plot(Ps,ModifiedFrequencies/analyt_Frequencies(1),'x','linewidth',3)
labels('$P/P_{Euler}$',' Frequency($P$)/Frequency(0) [Hz]')
grid on
set_graphics_defaults