%%  Fast Lagrangian top.
% 
% Fast-spinning Lagrange  top simulated by a beam model. The reference
% solution is available in Lagrange_top_fast (rigid-body solver).
function CBFastTop
    %%
    % Parameters:
    E=71240.0;%MPa
    nu=0.31;% Poisson ratio
    rho=2.7e-9;
    Width=60; Height=60; Length=4*Width;% cross-sectional dimensions and length of each leg in millimeters
    % Principal components of the tensor of inertia with respect to the origin of the coordinate system
    Mass=Width*Height*Length*rho;
    IL= (1/12)*Mass*(Width^2+Height^2);
    Ib= (1/12)*Mass*(Length^2+Height^2)+Mass*(Length/2)^2;
    Ih= (1/12)*Mass*(Width^2+Length^2)+Mass*(Length/2)^2;
    Omega0=313*pi;
    R0 = rotmat([0.05 0 0]);
    g=9.81e3; % 9.81 meters per second squared
    q= [0,0,-g*Width*Height*rho]';
    utol=1e-3;
    dt=min( [2*pi/norm(Omega0)/10, 0.005]);
    tend=0.8;
    Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    % Newmarkg = 0.6; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    maxit=30;
    graphics=0;
    camera=1.0e+003 *[0.0101   -0.0137    1.0393    0.0101 -0.0137    0.0000    0.0010         0  0    0.0111];
    Report =@(x) (disp(x) );
    % Choose the mass formulation:
    consistent_no_rotation_inertia=0;
    consistent_with_rotation_inertia=1;
    lumped_diagonal_no_rotation_inertia=2;
    lumped_diagonal_with_rotation_inertia=3;
    mass_type=1;

    %%
    % Mesh
    n=3;
    tolerance=Length/n/100;
    Parameters=Beam_cross_section(struct('type','square','a',Width));
    Parameters.x1x2_vector=[1 0 0];
    spin_vector = R0*[0 0 Omega0]';
    X=[0,0,0;(R0*[0,0,Length]')'];
    [fens,gcells] = Beam_member(X,n,@gcellset_beam3,Parameters);
    
    % The supported end is
    supportn=fenode_select(fens, struct('box', update_box([],X(1,:)), 'inflate', tolerance));
    % The Tip
    tipn=fenode_select(fens, struct('box', update_box([],X(2,:)), 'inflate', tolerance));
    
    %%
    % Properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells,...
        'mass_type',mass_type));
    
    %%
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u=0*geom0;
    Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    Rfield0=Rfield;
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    % Apply EBC's
    ebc_fenids=[supportn];
    ebc_prescribed=[1];
    ebc_comp=[1,2,3];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi = apply_ebc (dchi);
    % Number equations
    dchi = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    
    if (graphics)
        gv=graphic_viewer;
    else
        hold on;
        axis equal
        ;
    end
    
    % Now comes the nonlinear solution
    tup = 1;
    dchi = dchi*0; % zero out the displacement
    v = dchi*0; % zero out the Velocity
    xyz = gather(geom0,(1:count(fens)),'values','noreshape');
    vxyz = gather(v,(1:count(fens)),'values','noreshape');
    vxyz =[0*xyz, ones(count(fens),1)*spin_vector'];
    v= scatter (v,(1:count(fens)),vxyz);
    a = dchi*0; % zero out the Acceleration
    utol = 1e-13*get(dchi, 'neqns');
    
    K = start (sparse_sysmat, get(dchi, 'neqns'));
    M = start (sparse_sysmat, get(dchi, 'neqns'));
    G = start (sparse_sysmat, get(dchi, 'neqns'));
    
    fi = force_intensity(struct('magn' ,q));

    t=0; %
    step=0;
    while (t <= tend)
        t = t + dt;
        Report(['Time ' num2str(t) ]); % pause
        % Initialization
        dchi = apply_ebc(dchi);% Apply boundary conditions
        u1 = u; % guess
        Rfield1=Rfield; % guess
        stepdchi = 0*apply_ebc(dchi);% Total increment in current step
        a1 = -(1/Newmarkb/dt)*v -(1/2-Newmarkb)/Newmarkb*a;
        v1 = v + dt*((1-Newmarkg)*a + Newmarkg*a1);
        dchipv =gather_sysvec(dt*v +(dt^2/2*(1-2*Newmarkb))*a);
        vpv =gather_sysvec(v +(dt*(1-Newmarkg))*a);
        
        iter=1;
        while 1
            feb1=feb;
            F = start (sysvec, get(dchi, 'neqns'));
            F = assemble (F, distrib_loads_global(feb,geom0,u1,Rfield1,dchi, fi));
            [feb,evs] = restoring_force(feb,geom0,u1,Rfield1,dchi);       % Internal forces
            F = assemble (F, evs);
            %             K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi))); 
            K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi)));
            Km=get(K, 'mat');
            M =assemble (M,mass(feb,geom0,u1,Rfield1,dchi));
            Mm=get(M, 'mat');
            G =assemble (G,coriolis(feb,geom0,u1,Rfield1,v1,dchi));
            Gm=get(G, 'mat');
            rhs=get(F,'vec')...
                +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(stepdchi)+(1/(Newmarkb*dt^2))*dchipv)...
                +Gm*((-Newmarkg/Newmarkb/dt)*gather_sysvec(stepdchi)+(Newmarkg/Newmarkb/dt)*dchipv-vpv);
            dchi = scatter_sysvec(dchi, (Km+(Newmarkg/Newmarkb/dt)*Gm+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
            u1 = u1 + slice(dchi,(1:3),'transl');   % increment displacement
            stepdchi = stepdchi + dchi;
            v1 = v1 + (Newmarkg/Newmarkb/dt)*dchi;
            a1 = a1 + (1/Newmarkb/dt^2)*dchi;
            Rfield1=update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat'));
%             Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;% convergence check
            if (iter >maxit)% bailout for failed convergence
                error( [' Possible failed convergence']);
            end
            K = reinitialize (K);
            M = reinitialize (M);
            G = reinitialize (G);
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
        %         Report(['    Converged for t=' num2str(t)]); % pause 
        u = u1;       % update the displacement
        Rfield=Rfield1; % update the rotations
        v = v1;       % update the velocities
        a = a1;       % update the accelerations
        
            if graphics
            gv=reset(gv,[]);
            draw(feb, gv, struct ('x', geom0, 'u', 0*u1, 'R', Rfield0, 'facecolor','r'));
            draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
            %         draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'length',12.5));
                    camset(gv, camera);
            drawnow;
        else
           if (mod(step,20)==0)
               tipu=gather(u1,tipn, 'values', 'noreshape');
               tipx=X(2,:)+tipu;
               plot(tipx(1)/Length,tipx(2)/Length, 'ko', 'linewidth',2);
               xlabel('x-component of v_{CG}');
               ylabel('y-component of v_{CG}');
           end
            %             plot(t,tipu(1), 'rv');
            %             hold on
            %             plot(t,tipu(2), 'gv');
            %             plot(t,tipu(3), 'bv'); 
            figure(gcf);
        end
        step=step+1;
    end
    
end
