
%% Spinning shaft about an offset axis.
%
% Direct numerical simulation of a spinning shaft which is supported in
% bearings with an offset.
% 
function [rotary_speeds,umnum,umana]=CB_spinning_shaft
    % Parameters:
    E=30002*1000;%Young's modulus 30002ksi
    nu=0.3;% Poisson ratio
    radius_internal=2.5/2;%in
    radius_external=6.25/2;%in
    L=45*12;% in
    g=32.17*12; % in/second^2
    rho=490/12^3/g; %lb/ft^3
    eccentricity=0.1;% axis eccentricity in inches
    zeta= 0.05; % damping ratio for mode 1
    zeta_1= zeta; % damping ratio for mode 1
    zeta_2= zeta; % damping ratio for mode 2
    % Stiffness proportional damping is selected.
    omega_1=2*pi*1.832; % computed previously for this structure in stival_4_7
    omega_2=2*pi*7.328; % computed previously for this structure in stival_4_7
    % zeta = (Rayleigh_damping_stiffness/omega_1)/2
    Rayleigh_damping_mass= 1/(omega_2^2-omega_1^2)*[omega_2^2,-omega_1^2]*2*[zeta_1*omega_1;zeta_2*omega_2];
    Rayleigh_damping_stiffness= 1/(omega_2^2-omega_1^2)*[-1,1]*2*[zeta_1*omega_1;zeta_2*omega_2];
    
    % Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    Newmarkg = 0.8; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    maxit=70;
    
    
    %%
    % Output control
    Report =@(x) (disp(x) );
    graphics= false;
    
    X=[[-L/2 eccentricity 0];[L/2 eccentricity 0]];
    
    %%
    % Cross-sectional properties
    Parameters=Beam_cross_section(struct('type','hollow_circle','Rint',radius_internal,'Rext',radius_external));
    Parameters.x1x2_vector=[0 1 0];
    
    
    %%
    %
    n=2;
    [fens,gcells] = Beam_member(X,2*n,@gcellset_beam3,Parameters);
    midn=fenode_select(fens, struct('box', update_box([],mean(X))));
    end1n=fenode_select(fens, struct('box', update_box([],X(1,:))));
    end2n=fenode_select(fens, struct('box', update_box([],X(2,:))));
    
    %%
    % Material properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells,...
        'Rayleigh_damping_stiffness',Rayleigh_damping_stiffness,...
        'Rayleigh_damping_mass',Rayleigh_damping_mass));
    
    rotary_speeds=25:25:200;% This can be used to explore the range of RPMs more fully
    rotary_speeds=[25,115,200];
    
    umnum=[];
    umana=[];
    for rotary_speed=rotary_speeds;%rotary speed, RPM
        freq=rotary_speed/60;% circular frequency in hertz
        Omega=freq*2*pi;% Angular frequency of spin
        
        %%
        % Time integration parameters
        Tper=1/freq;
        dt=Tper/41.4;
        tend=(6+(rotary_speed/200)*50)*Tper;
        
        
        spin_vector= [Omega;0;0];
        spin_center= [0 0 0];
        
        %%
        % Construct the requisite fields, geometry and displacement
        % Initialize configuration variables
        geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
        u=0*geom0;
        Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
        Rfield0=Rfield;
        % Define the displacement increment field
        dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

        % Apply EBC's
        dchi  = set_ebc(dchi, end1n, 1, [1,2,3,4], 0);
        dchi  = set_ebc(dchi, end2n, 1, [1,2,3,4], 0);
        dchi  = apply_ebc (dchi);
        
        % Number equations
        dchi  = numbereqns (dchi);
        neqns=get(dchi, 'neqns');
        
        
        if graphics, gv=graphic_viewer; end
        
        if graphics,
            gv=reset(clear(gv,[]),[]);
            %                     camset(gv,cam);
            view(2);
            displ=slice(dchi,(1:3),'displ');
            rot=slice(dchi,(4:6),'rot');
            draw(feb, gv, struct ('x', geom, 'u', displ, 'rot', rot, 'facecolor','none'));
            pause(0.5);
            %                     cam =camget(gv);
        end
        
        % initial conditions
        v = dchi*0; % zero out the Velocity
        xyz = gather(geom0,(1:count(fens)),'values','noreshape');
        vxyz = gather(v,(1:count(fens)),'values','noreshape');
        vxyz =[-xyz*skewmat(spin_vector), ones(count(fens),1)*spin_vector'];
        v= scatter (v,(1:count(fens)),vxyz);
        a = dchi*0; % zero out the Acceleration
        utol = 1e-9*get(dchi, 'neqns');
        
        K = start (sparse_sysmat, get(dchi, 'neqns'));
        M = start (sparse_sysmat, get(dchi, 'neqns'));
        C = start (sparse_sysmat, get(dchi, 'neqns'));
        
        Mid_uy=[];
        
        t=0; % time=load magnitude
        while (t <= tend)
            t = t + dt;
            Report(['Time ' num2str(t) ]); % pause
            % Initialization
            dchi = apply_ebc(dchi);% Apply boundary conditions
            u1=u;%
            end1nu1=(rotmat(spin_vector*t)-eye(3))*(X(1,:)-spin_center)' ;
            u1  = set_ebc(u1, end1n, 1, 1:3, [end1nu1']);
            u1  = set_ebc(u1, end2n, 1, 1:3, [end1nu1']);
            u1 = apply_ebc(u1);
            Rfield1=Rfield;
            dro1 = 0*dchi;
            dro1  = set_ebc(dro1, end1n, 1, 4:6, [spin_vector'*dt]);
            dro1  = set_ebc(dro1, end2n, 1, 4:6, [spin_vector'*dt]);
            dro1 = apply_ebc(dro1);
            Rfield1=update_rotation_field(Rfield1,slice(dro1,(4:6),'rot'));
            stepdchi = 0*apply_ebc(dchi);% Total increment in current step
            a1 = -(1/Newmarkb/dt)*v -(1/2-Newmarkb)/Newmarkb*a;
            v1 = v + dt*((1-Newmarkg)*a + Newmarkg*a1);
            dchipv =gather_sysvec(dt*v +(dt^2/2*(1-2*Newmarkb))*a);
            vpv =gather_sysvec(v +(dt*(1-Newmarkg))*a);
            
            iter=1;
            while 1
                feb1=feb;
                F = start (sysvec, get(dchi, 'neqns'));
                [feb,evs] = restoring_force(feb,geom0,u1,Rfield1,dchi);       % Internal forces
                F = assemble (F, evs);
                K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi)));
                Km=get(K, 'mat');
                M =assemble (M,mass(feb,geom0,u1,Rfield1,dchi));
                Mm=get(M, 'mat');
                C =assemble (C,damping_proportional(feb,geom0,u1,Rfield1,dchi));
                Cm=get(C, 'mat');
                rhs=get(F,'vec')...
                    +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(stepdchi)+(1/(Newmarkb*dt^2))*dchipv)...
                    +Cm*((-Newmarkg/Newmarkb/dt)*gather_sysvec(stepdchi)+(Newmarkg/Newmarkb/dt)*dchipv-vpv);
                dchi = scatter_sysvec(dchi, (Km+(Newmarkg/Newmarkb/dt)*Cm+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
                u1 = u1 + slice(dchi,(1:3),'transl');   % increment displacement
                stepdchi = stepdchi + dchi;
                v1 = v1 + (Newmarkg/Newmarkb/dt)*dchi;
                a1 = a1 + (1/Newmarkb/dt^2)*dchi;
                Rfield1=update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat'));
                Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
                if (max(abs(gather(dchi,[],'values'))) < utol) break; end;% convergence check
                if (iter >maxit)% bailout for failed convergence
                    warning( [' Possible failed convergence']);
                end
                K = reinitialize (K);
                M = reinitialize (M);
                C = reinitialize (C);
                iter=iter+1;
            end
            [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);         % final update
            Report(['    Converged for t=' num2str(t)]); % pause
            u = u1;       % update the displacement
            Rfield=Rfield1; % update the rotations
            v = v1;       % update the velocities
            a = a1;       % update the accelerations
            
            umid=gather(u,midn,'values');
            Mid_uy=[Mid_uy;umid(2:3)'-end1nu1(2:3)'];
            
            if graphics
                displ=slice(ur,(1:3),'displ');
                rot=slice(ur,(4:6),'rot');
                gv=reset(gv,[]);
                draw(feb, gv, struct ('x', geom0, 'u', 0*u1, 'R', Rfield0, 'facecolor','r'));
                draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
                draw_local_coordinates(feb, gv, struct ('x', geom, 'u', displ, 'R', Rfield1, 'length',12.5));
                draw_axes (gv,struct('length',L/30 ));
                camset(gv, 1.0e+003 *[-4.7025   -0.1968    0.1667    0.0018    0.0086    0.0022         0         0    0.0010    0.0004]);
                drawnow;
            else
                if (mod(size(Mid_uy,1),2)==0)
                    %                     plot(t/Tper,Mid_uy(end,1),'m+'); hold on
                    %                     plot(t/Tper,Mid_uy(end,2),'bx');
                                        hold on 
                    plot(Mid_uy(end,1),Mid_uy(end,2),'bx'); hold on
                    title(['rotary speed ' num2str(rotary_speed)])
                    drawnow;
                end
            end
        end
        
        %done for the rotary speed
        umnum=[umnum,max([abs(Mid_uy(end-3*round(Tper/dt):end,1));abs(Mid_uy(end-3*round(Tper/dt):end,2))])]
        umana=[umana,uampl_analyt(rotary_speed/60*2*pi,omega_1,eccentricity)];
    end
    
    %%
    % The analytical solution is in the form of (4.58).
    function v=uampl_analyt(Omega,omega_1,eccentricity)
        e=4*eccentricity/pi;
        beta=Omega./omega_1;
        v=e*beta.^2./sqrt((1-beta.^2).^2+(2*zeta*beta).^2);
    end 
    %%
    % The numerical solution matches the analytical predictions fairly well.
    % Note that the numerical model is fully nonlinear and  the
    % resonant frequency is therefore shifted a little bit higher than predicted by the
    % linear model.
    rss=linspace(0, 200, 200);
    figure;
    plot(rss,uampl_analyt(rss/60*2*pi,omega_1,eccentricity),'r-','linewidth',3)
    hold on
    plot(rotary_speeds,umnum,'x','linewidth',3)
    labels('Rotary speed [rpm]','Maximum displacement [in]')
    legend({' Formula ',' Numerical'})
    grid on
    set_graphics_defaults
end