% Multi-physics simulation of a micro-actuator
% The micro-actuator analyzed is a variation on the actuator described in:
% A reliable single-layer out-of-plane micro-machined thermal actuator, W-C
% Chen et al., Sensors and Actuators A 103 (2003) 48-58. 
% 
[fens,gcells] =actuator1_mesh;

geom = field(struct('name',['geom'], 'dim', 3, 'fens',fens));
tol=1e-9;

% Electrostatic problem
DV=5;% voltage drop in volt
el_conductivity = 1/1.1e-5;% Ohm m

elprop=property_electrostatics(struct('electrical_conductivity',el_conductivity));
elmater=mater_electrostatics (struct('property',elprop));
elfeb = feblock_electrostatics (struct ('mater',elmater,...
    'gcells',gcells,'integration_rule',gauss_rule(3,2)));
phi=field(struct('name',['phi'], 'dim', 1, 'nfens',get(geom,'nfens')));
fenids=fenode_select(fens,struct('box',[0,15e-6,0,0,0,2e-6],...
    'inflate', tol)) ; % prescribed Voltage on substrate
prescribed=fenids*0+1;
comp=[];
val=zeros(length(fenids),1)+DV/2;
phi = set_ebc(phi, fenids, prescribed, comp, val);
fenids=fenode_select(fens,struct('box',[0,0,-inf,inf,-inf,inf],...
    'inflate', tol)) ; % zero Voltage on the plane of symmetry
prescribed=fenids*0+1;
comp=[];
val=zeros(length(fenids),1)+0;
phi = set_ebc(phi, fenids, prescribed, comp, val);
phi = apply_ebc (phi);
phi = numbereqns (phi);
    
Ke = start (sparse_sysmat, get(phi, 'neqns'));
Ke = assemble (Ke, electrical_conductivity(elfeb, geom, phi, []));
Fe = start (sysvec, get(phi, 'neqns'));
Fe = assemble (Fe, nz_ebc_loads_electrical_conductivity(elfeb, geom, phi, []));
phi = scatter_sysvec(phi, Ke\Fe);

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
V=get(phi,'values');
cmap= cadcolors2;
dcm=data_colormap(struct('range',[min(V),max(V)],'colormap',cmap));
colorfield=field(struct ('name', ['colorfield'], 'data',...
    map_data(dcm, V)));
draw(elfeb, gv, struct ('x',geom, 'u',0*geom,...
    'colorfield',colorfield, 'shrink',1));
draw_colorbar(gv, struct('colormap',cmap,'label','Voltage',...
    'minmax',get(dcm ,'minmax')))
rotate(gv);

% Thermal analysis
kappa=157*eye(3, 3); % W, conductivity matrix
T_substrate=293;% substrate temperature in degrees Kelvin

thprop=property_diffusion(struct('conductivity',kappa));
thmater=mater_diffusion (struct('property',thprop));
thfeb = feblock_diffusion (struct ('mater',thmater,...
    'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));
theta=field(struct('name',['theta'], 'dim', 1, 'nfens',get(geom,'nfens')));
fenids=fenode_select(fens,struct('box',[-inf,inf,0,0,-inf,inf],...
    'inflate', tol)) ; %  prescribed temperature on substrate
prescribed=fenids*0+1;
comp=[];
val=zeros(length(fenids),1)+T_substrate;
theta = set_ebc(theta, fenids, prescribed, comp, val);
theta = apply_ebc (theta);
theta = numbereqns (theta);

K = start (sparse_sysmat, get(theta, 'neqns'));
K = assemble (K, conductivity(thfeb, geom, theta));
F = start (sysvec, get(theta, 'neqns'));
F = assemble (F, resistive_heating_loads(elfeb, geom, theta, phi));
F = assemble (F, nz_ebc_loads_conductivity(thfeb, geom, theta));
theta = scatter_sysvec(theta, get(K,'mat')\get(F,'vec'));

gv=graphic_viewer;
gv=reset (gv,[]);
T=get(theta,'values');
cmap= hot;
dcm=data_colormap(struct('range',[min(T),max(T)],'colormap',cmap));
colorfield=field(struct ('name', ['colorfield'], 'data',...
    map_data(dcm, T)));
draw(thfeb, gv, struct ('x',geom, 'u',0*geom,...
    'colorfield',colorfield, 'shrink',1));
draw_colorbar(gv, struct('colormap',cmap,'label','Temperature',...
    'minmax',[min(T),max(T)]))
rotate(gv);

% Mechanical analysis
E= 169000000;
nu =0.278;
alpha = 2.5e-6;
% Material: mechanical
mechprop = property_linel_iso (struct ('E',E,...
    'nu',nu,'alpha', alpha));
mechmater=mater_defor_ss_linel_triax(struct('property',mechprop));

% Finite element block: mechanical
mechfeb = feblock_defor_ss (struct ('mater',mechmater, 'gcells',gcells, ...
    'integration_rule',gauss_rule(3,2)));

u=0*geom;
fenids=fenode_select(fens,struct('box',[-inf,inf,0,0,-inf,inf],...
    'inflate', tol)) ; %  prescribed zero displacement on substrate
 % prescribed displacement
prescribed=fenids*0+1;
comp=[];
val=zeros(length(fenids),1)+0;
u = set_ebc(u, fenids, prescribed, comp, val);
fenids=fenode_select(fens,struct('box',[0,0,-inf,inf,-inf,inf],...
    'inflate', tol)) ;  % prescribed displacement on plane of symmetry
prescribed=fenids*0+1;
comp=fenids*0+1;% x-compe
val=zeros(length(fenids),1)+0;
u = set_ebc(u, fenids, prescribed, comp, val);
u = apply_ebc (u);

u = numbereqns (u);
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(mechfeb, geom, u));
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, thermal_strain_loads(mechfeb, geom, u, theta-T_substrate));
u = scatter_sysvec(u, get(K,'mat')\get(F,'vec'));

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
draw(mechfeb, gv, struct ('x',geom, 'u',0*u,...
    'facecolor','none'));
U=get(u,'values');
Uz=U(:,3);
cmap= jet;
dcm=data_colormap(struct('range',[min(Uz),max(Uz)],'colormap',cmap));
colorfield=field(struct ('name', ['colorfield'], 'data',...
    map_data(dcm, Uz)));
draw(mechfeb, gv, struct ('x',geom, 'u',u,...
    'colorfield',colorfield));
draw_colorbar(gv, struct('colormap',cmap,'label','Uz',...
    'minmax',[min(Uz),max(Uz)]))

rotate(gv);
