% General smoothing of meshes.
%
% function v =smooth_mesh(t,v,ifixedv,options) 
%
% v=vertex coordinates
% t=cell connectivities, indexes into v  
% visfixed=Fixed or free vertex? Array of the same dimension as size(v,1).
%     fixed (immovable) vertices -- visfixed(k)~=0
%     free (movable) vertices -- visfixed(k)==0
% options= struct() with optional fields
%     method='laplace' or 'taubin'
%     npass=how many passes of smoothing? (Default 2)
function v =smooth_mesh(t,v,visfixed,options)
    inputv=v; %Save the input coordinates
    npass =2;
    method =@taubin_smoother;
    if (exist('options','var'))
        if (isfield(options,'npass'))
            npass= options.npass;
        end
        if (isfield(options,'method'))
            if ( strcmp(options.  method,'laplace'))
                method =@laplace_smoother;
            elseif ( strcmp(options.  method,'taubin'))
                method =@taubin_smoother;
            end
        end

    end

    
    % find neighbors for the volume connections
    vn = vertex_neighbors([],t,v);

    % Smoothing considering all connections through the volume
    v =  method(v,vn,visfixed,npass,0.5,-0.5);
end

function nv =  taubin_smoother(v,vn,visfixed,npass,lambda,mu)
    nv=v;
    for I= 1:npass
        o=randperm(length(vn));
        damping_factor=lambda;
        for k= 1:length(vn)
            r=o(k);
            n=setdiff(vn{r},r);
            if (~isempty(n)) && (~visfixed(r))
                nv(r,:)=(1-damping_factor)*v(r,:)+ damping_factor*sum(v(n,:))/length(n);
            end
        end
        v=nv;
        damping_factor=mu;
        for k= 1:length(vn)
            r=o(k);
            n=setdiff(vn{r},r);
            if (~isempty(n)) && (~visfixed(r))
                nv(r,:)=(1-damping_factor)*v(r,:)+ damping_factor*sum(v(n,:))/length(n);
            end
        end
        v=nv;
    end
end

function nv =  laplace_smoother(v,vn,visfixed,npass,lambda,mu)
    damping_factor=lambda;
    for I= 1:npass
        nv=v;
        o=randperm(length(vn));
        for k= 1:length(vn)
            r=o(k);
            n=setdiff(vn{r},r);
            if (~isempty(n)) && (~visfixed(r))
                nv(r,:)=damping_factor*v(r,:)+ (1-damping_factor)*sum(v(n,:))/length(n);
            end
        end
        v=nv;
    end
end


function vn =  vertex_neighbors(vn,f,v)
    if (isempty(vn))
        vn=cell(1,size(v,1));
    end
    for I= 1:size(f,1)
        for r= 1:size(f,2)
            vn{f(I,r)}=[vn{f(I,r)},f(I,:)];
        end
    end
    for I= 1:size(vn,1)
        vn{I}=unique(vn{I});
    end
end